#include "hw_impl_zybo.h"
#include "type_def.h"
#include "iic_utils.h"
#include <stdio.h>

/**
 * Test for the LEDDriver and SystemDriver.
 *
 * This is essentially a basic "blink" program, using the mio7 LED
 * on the Zybo board.
 *
 * Instructions:
 * 1) Connect Zybo board to computer by USB cable.
 * 2) Set the RUN_TESTS macro in main.c
 * 3) Uncomment only this test in main.c
 * 4) Run main.c
 * 5) Observe MIO7 LED on board blinking at 1 second intervals.
 */
int test_zybo_mio7_led_and_system() {
  struct LEDDriver mio7 = create_zybo_mio7_led();
  struct SystemDriver sys = create_zybo_system();
  mio7.reset(&mio7);
  sys.reset(&sys);

  while (1) {
    mio7.turn_on(&mio7);
    sys.sleep(&sys, 1000000);
    mio7.turn_off(&mio7);
    sys.sleep(&sys, 1000000);
  }

  return 0;
}

/**
 * Tests for the I2CDriver (one for each I2C device we use)
 *
 * Instructions:
 * 1) Connect Zybo Board to computer by USB cable.
 * 2) Prepare a breadboard to accomplish the following connections:
 *   - Zybo GND     <-> LIDAR GND
 *   - Zybo WALL    <-> LIDAR VCC
 *   - Zybo VV5V0   <-> LIDAR VCC
 *   - Zybo SDA/SCL <-> LIDAR SDA/SCL
 *     - The Zybo I2C SCL and SDA pins are on pins 1 and 2 of PORT JF, respectively
 * 3) Place breakpoint in this test function, somewhere in the while loop.
 * 4) Set the RUN_TESTS macro in main.c
 * 5) Uncomment only this test in main.c
 * 6) Debug main.c
 * 7) Step through the while loop, observing x change as you physically change the
 *    distance of the LIDAR sensor.
 */
int test_zybo_i2c() {
  struct I2CDriver i2c = create_zybo_i2c();
  struct SystemDriver sys = create_zybo_system();
  i2c.reset(&i2c);
  sys.reset(&sys);

  lidar_t lidar = { };
  iic_set_globals(&i2c, &sys);
  if (iic0_lidarlite_init()) return 0;
  short x;
  while (1) {
    iic0_lidarlite_read_distance(&lidar);
    x = lidar.distance_cm;
  }
  return 0;
}

int test_zybo_i2c_imu() {
  struct I2CDriver i2c = create_zybo_i2c();
  struct SystemDriver sys = create_zybo_system();
  i2c.reset(&i2c);
  sys.reset(&sys);

  gam_t gam = { };
  iic_set_globals(&i2c, &sys);
  if (iic0_mpu9150_start()) return 0;
  short x;
  while (1) {
    iic0_mpu9150_read_gam(&gam);
  }
  return 0;
}

int test_zybo_i2c_px4flow() {
  struct I2CDriver i2c = create_zybo_i2c();
  struct SystemDriver sys = create_zybo_system();
  i2c.reset(&i2c);
  sys.reset(&sys);

  px4flow_t of;
  iic_set_globals(&i2c, &sys);

  if(iic0_px4flow_init(&of)) return 0;

  for(;;) {
    unsigned int delay = 5;

    sys.sleep(&sys, delay * 1000);

    iic0_px4flow_update(&of, delay / 1000.);
  }
  return 0;
}

int test_zybo_i2c_all() {
  struct I2CDriver i2c = create_zybo_i2c();
  struct SystemDriver sys = create_zybo_system();
  i2c.reset(&i2c);
  sys.reset(&sys);

  lidar_t lidar = { };
  px4flow_t of = { };
  gam_t gam = { };
  iic_set_globals(&i2c, &sys);

  if (iic0_px4flow_init(&of)) return 0;
  if (iic0_mpu9150_start()) return 0;
  if (iic0_lidarlite_init()) return 0;

  int lidarErrors = 0;
  int gamErrors = 0;
  int nLoops = 0;
  int of_errors = 0;

  for(;;) {
    unsigned int delay = 5;

    sys.sleep(&sys, delay * 1000);

    if (iic0_px4flow_update(&of, delay / 1000.)) {
    	of_errors += 1;
    }

    iic0_mpu9150_read_gam(&gam);
    iic0_lidarlite_read_distance(&lidar);

    if (lidar.distance_cm > 5000) {
    	lidarErrors += 1;
    }
    if (gam.accel_z > -0.8) {
    	gamErrors += 1;
    }
    nLoops += 1;
  }
  return 0;
}

/**
 * Test for the PWMInputDriver.
 *
 * Instructions:
 * 1) Connect the quad Zybo board to computer using USB.
 * 2) Move jumper on Zybo board to use JTAG instead of SD.
 * 3) Turn on Zybo board and turn on Spektrum handheld controller.
 *   - Verify receiver on quad pairs with controller (orange LED should turn on)
 * 3) Place breakpoint somewhere in the while loop of this function.
 * 4) Set the RUN_TESTS macro in main.c
 * 5) Uncomment only this test in main.c
 * 6) Debug main.
 * 7) Observe the values of pwm_inputs in debugger chaning as you use the
 *    Spektrum RC controller.
 */
int test_zybo_pwm_inputs() {
  struct PWMInputDriver pwm_inputs = create_zybo_pwm_inputs();
  pwm_inputs.reset(&pwm_inputs);

  unsigned long pwms[6];
  while (1) {
    int i;
    for (i = 0; i < 6; i += 1) {
      pwm_inputs.read(&pwm_inputs, i, &pwms[i]);
    }
    continue;
  }
}

/**
 * Test for the PWMOutputDriver.
 *
 * Instructions:
 * 1) Connect the quad Zybo board to computer using USB.
 * 2) Move jumper on Zybo board to use JTAG instead of SD.
 * 3) Get an oscilloscope and observe PMOD outputs JE7-JE10
 * 4) Set the RUN_TESTS macro in main.c
 * 5) Uncomment only this test in main.c
 * 6) Run main.
 * 7) Observe the PWM width of those PMOD pins changing with time
 */
int test_zybo_pwm_outputs() {
  struct PWMOutputDriver pwm_outputs = create_zybo_pwm_outputs();
  pwm_outputs.reset(&pwm_outputs);
  struct SystemDriver sys = create_zybo_system();
  sys.reset(&sys);

  while (1) {
    int j;
    for (j = 100000; j < 200000; j += 1) {
      int i;
      for (i = 0; i < 4; i += 1) {
        pwm_outputs.write(&pwm_outputs, i, j);
        sys.sleep(&sys, 50);
      }
    }
    continue;
  }
  return 0;
}

/**
 * Test for the AXI timer, using LEDDriver.
 *
 * This is essentially a basic "blink" program, using the mio7 LED
 * on the Zybo board.
 *
 * Instructions:
 * 1) Connect Zybo board to computer by USB cable.
 * 2) Set the RUN_TESTS macro in main.c
 * 3) Uncomment only this test in main.c
 * 4) Run main.c
 * 5) Observe MIO7 LED on board blinking at 1 second intervals.
 */
int test_zybo_axi_timer() {
  struct TimerDriver axi = create_zybo_axi_timer();
  struct LEDDriver led = create_zybo_mio7_led();
  axi.reset(&axi);
  led.reset(&led);

  unsigned long time;

  while (1) {
    axi.restart(&axi);
    time = 0;
    while (time < 1000000) {
      axi.read(&axi, &time);
    }
    led.turn_off(&led);
    while (time < 2000000) {
      axi.read(&axi, &time);
    }
    led.turn_on(&led);
  }
}

/**
 * Test for the Global timer, using LEDDriver.
 *
 * This is essentially a basic "blink" program, using the mio7 LED
 * on the Zybo board.
 *
 * Instructions:
 * 1) Connect Zybo board to computer by USB cable.
 * 2) Set the RUN_TESTS macro in main.c
 * 3) Uncomment only this test in main.c
 * 4) Run main.c
 * 5) Observe MIO7 LED on board blinking at 1 second intervals.
 */
int test_zybo_global_timer() {
  struct TimerDriver global = create_zybo_global_timer();
  struct LEDDriver led = create_zybo_mio7_led();
  global.reset(&global);
  led.reset(&led);

  unsigned long time;

  while (1) {
    global.restart(&global);
    time = 0;
    while (time < 1000000) {
      global.read(&global, &time);
    }
    led.turn_off(&led);
    while (time < 2000000) {
      global.read(&global, &time);
    }
    led.turn_on(&led);
  }
}

/**
 * Test for the UARTDriver.
 *
 * Instructions:
 * 1) Connect Zybo board to computer by USB cable.
 * 2) Get a FTDI Basic Sparkfun board in order to connect the UART pins
 *    on the Zybo to the computer by USB.
 *    - Zybo PMOD JC2 (TX) <-> Sparkfun Board RX
 *    - Zybo PMOD JC3 (RX) <-> sparkfun Board Tx
 *    - Zybo PMOD JC5 (GND) <-> Sparkfun Board GDN
 * 2) Set the RUN_TESTS macro in main.c
 * 3) Uncomment only this test in main.c
 * 4) Run main.c
 * 5) Execute quad/scripts/tests/test_zybo_uart.py
 *    - Observe test results on terminal
 *    - You might be able to see LED MIO7 blink when it receives bytes
 */
int test_zybo_uart() {
  struct UARTDriver uart = create_zybo_uart();
  struct LEDDriver led = create_zybo_mio7_led();
  uart.reset(&uart);
  led.reset(&led);

  unsigned char c;
  while (1) {
    if (uart.read(&uart, &c)) {
      // read failed
      led.turn_off(&led);
    } else {
      // read successful
      led.turn_on(&led);
      uart.write(&uart, c);
    }
  }

  return 0;
}


